using System;
using OpenNETCF.Win32;
using System.Collections;

namespace LocationBasedMessenger
{
		/// <summary>
		/// Summary description for Settings.
		/// </summary>
		public class Settings
		{
			RegistryKey reg;
			const string kkey=@"SOFTWARE\Binary Refinery\GPS";
			public Settings()
			{
				reg = Registry.LocalMachine.OpenSubKey(kkey,true);
				if (reg==null)
				{
					Registry.LocalMachine.CreateSubKey(kkey);
					reg = Registry.LocalMachine.OpenSubKey(kkey,true);
				}
			}
			public string ServiceRefreshTime
			{
				get
				{
					return RegReadDefault("ServiceRefreshTime","30").ToString();
				}
				set
				{
					RegWrite("ServiceRefreshTime",value);
				}
			}

			public string LocationServiceAdress
			{
				get
				{
					return RegReadDefault("LocationServiceAdress","http://maniac.rz.tu-ilmenau.de/LocationService/LocationService.asmx").ToString();
				}
				set
				{
					RegWrite("LocationServiceAdress",value);
				}
			}
			
			public string BaudRate
			{
				get
				{
					return RegReadDefault("baudrate","4800").ToString();
				}
				set
				{
					RegWrite("baudrate",value);
				}

			}

			public string ComPort
			{
				get
				{
					return RegReadDefault("comport","COM7:").ToString();
				}
				set
				{
					RegWrite("comport",value);
				}

			}
			public bool AutoDiscovery
			{
				get
				{
					return Convert.ToInt32(RegReadDefault("autodiscovery",1).ToString())==1;
				}
				set
				{
					RegWrite("autodiscovery",value?1:0);
				}
			}


			public bool TraceOn
			{
				get
				{
					return Convert.ToInt32(RegReadDefault("traceon",0).ToString())==1;
				}
				set
				{
					RegWrite("traceon",value?1:0);
				}

			}

			public bool DemoModeOn
			{
				get
				{
					return Convert.ToInt32(RegReadDefault("demomodeon",0).ToString())==1;
				}
				set
				{
					RegWrite("demomodeon",value?1:0);
				}

			}

			public string TraceFile
			{
				get
				{
					return (RegReadDefault("tracefile",@"\gpslog.txt").ToString());
				}
				set
				{
					RegWrite("tracefile",value);
				}

			}

			public string DemoFile
			{
				get
				{
					return (RegReadDefault("demofile",@"gpsdemo.txt").ToString());
				}
				set
				{
					RegWrite("demofile",value);
				}

			}
			
			public string[] COMDevices()
			{
				// lists all modems on the device
				ArrayList al = new ArrayList();
				RegistryKey defkey;
				string[] keyNames;
				string keyvalue="";
				string strport="";
				string strprefix="";
			
			
		
				// now get the active
				defkey=Registry.LocalMachine.CreateSubKey(@"Drivers\Active");
			
				keyNames =defkey.GetSubKeyNames();
				foreach (string x in keyNames)
				{
					// only get names that don't start with
					try	
					{
						keyvalue=defkey.CreateSubKey(x).GetValue("Key").ToString();
						strport=defkey.CreateSubKey(x).GetValue("Name").ToString();
					
						// fudge to remove rubbish off the end of the string
						strport = strport.Substring(0,strport.IndexOf("'"));

						RegistryKey thekey=Registry.LocalMachine.CreateSubKey(keyvalue);
					
						strprefix = thekey.GetValue("prefix").ToString().ToUpper();
						if (strprefix.StartsWith("COM"))
						{
							//string strname=thekey.GetValue("FriendlyName").ToString();
							//al.Add(strport+"|"+strname);
							al.Add(strport);
						}
					}
					catch
					{

					}
				}

				return (string[]) al.ToArray(typeof(string));
			}


	
			public string[] ModemList()
			{
				// lists all modems on the device
				RegistryKey defkey=Registry.LocalMachine.CreateSubKey("ExtModems");
			
				string[] keyNames =defkey.GetSubKeyNames();
				ArrayList al = new ArrayList();
				foreach (string x in keyNames)
				{
					// only get names that don't start with
					if (!x.StartsWith("\'"))
					{
						string strname=defkey.CreateSubKey(x).GetValue("FriendlyName").ToString();
						string strport=defkey.CreateSubKey(x).GetValue("Port").ToString();
			
						al.Add(strport+"|"+strname);
					}
				}
				return (string[]) al.ToArray(typeof(string));
			}



			private object RegReadDefault(string key, object val)
			{
				if (reg.GetValue(key)==null)
				{
					reg.SetValue(key,val);
				}
				return reg.GetValue(key);
			}

			private void RegWrite(string key, object val)
			{
				reg.SetValue(key,val);
			}
		
		}
	}