using System;
using OpenNETCF.IO.Serial;
using OpenNETCF.Win32;

using System.Text;
using System.Collections;
using System.Threading;
using System.IO;
using System.Globalization;

/// <summary>
/// Produced Under the Terms Of The OpenNETCF License
/// 
/// By Richard Jones - Richard@BinaryRefinery.com
/// 
/// </summary>
namespace OpenNETCF.IO.Serial.GPS
{
	public class GPS
	{
		#region private members
		Port cp;
		
		BaudRates baudrate=BaudRates.CBR_4800;
		string comport="COM1:";

		string strreceived="";
		int sentencecount=0;
		string storedgprmc="";
		string storedgpgga="";
		string storedgpsa="";
		string storedgpgsv="";
		string storedgpgll="";
		string storedgpvtg="";

		private States state=States.Stopped;
		private bool autodiscovery=false;
		private Thread thrd;


		private bool traceon=false;
		private string tracefile=@"\gpslog.txt";

		private bool demomodeon=false;
		private string demofile=@"gpsdemo.txt";
		string lasterror="";
		
		private objPosition waypoint=null;
		decimal distancetowaypoint=0;
		decimal bearingtowaypoint=0;
		decimal coursecorrection=0;

		// Position dilution of precision
		decimal pdop=0;

		// Horizontal dilution of precision
		decimal hdop=0;

		// Vertical dilution of precision
		decimal vdop=0;

		
		//Maximum Allowable HDOP Error
		decimal hdopmaxerror=6;


		#endregion

		#region public members

		public void DemoIt()
		{
			reset_gps_vars();



			string gpsdata="";

			gpsdata="$GPGLL,4916.452349,N,12311.123215,W,225444.00,A,A*6A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051936,A*30";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051938,A*3E";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051940,A*31";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051942,A*33";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051944,A*35";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051946,A*37";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051948,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051950,A*30";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051952,A*32";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051954,A*34";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051956,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.015,N,07953.247,W,051958,A*38";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.048,N,07953.247,W,052000,A*37";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.096,N,07953.247,W,052002,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.144,N,07953.247,W,052004,A*3E";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.117,N,07953.323,W,052006,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.103,N,07953.386,W,052008,A*3D";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.088,N,07953.449,W,052010,A*32";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.073,N,07953.513,W,052012,A*3A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.058,N,07953.576,W,052014,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.043,N,07953.639,W,052016,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.028,N,07953.702,W,052018,A*3C";
			nmea(gpsdata);

			gpsdata="$GPGLL,4339.013,N,07953.766,W,052020,A*3D";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.998,N,07953.829,W,052022,A*30";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.983,N,07953.892,W,052024,A*3C";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.968,N,07953.956,W,052026,A*32";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.953,N,07954.019,W,052028,A*31";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.938,N,07954.082,W,052030,A*37";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.923,N,07954.145,W,052032,A*35";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.908,N,07954.209,W,052034,A*31";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.893,N,07954.272,W,052036,A*3C";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.878,N,07954.335,W,052038,A*35";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.863,N,07954.399,W,052040,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.848,N,07954.462,W,052042,A*3E";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.833,N,07954.525,W,052044,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.818,N,07954.588,W,052046,A*3A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.803,N,07954.652,W,052048,A*3A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.788,N,07954.715,W,052050,A*3D";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.773,N,07954.778,W,052052,A*30";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.758,N,07954.841,W,052054,A*3A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.743,N,07954.905,W,052056,A*33";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.728,N,07954.968,W,052058,A*3B";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.713,N,07955.031,W,052100,A*3B";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.698,N,07955.095,W,052102,A*35";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.683,N,07955.158,W,052104,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.668,N,07955.221,W,052106,A*33";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.653,N,07955.284,W,052108,A*3A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.638,N,07955.348,W,052110,A*3F";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.623,N,07955.411,W,052112,A*3C";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.609,N,07955.474,W,052114,A*31";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.594,N,07955.537,W,052116,A*32";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.579,N,07955.601,W,052118,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.564,N,07955.664,W,052120,A*3D";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.549,N,07955.727,W,052122,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.534,N,07955.791,W,052124,A*37";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.519,N,07955.854,W,052126,A*3C";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.504,N,07955.917,W,052128,A*38";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.489,N,07955.980,W,052130,A*3B";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.474,N,07956.044,W,052132,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.459,N,07956.107,W,052134,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.444,N,07956.170,W,052136,A*38";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.429,N,07956.233,W,052138,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.414,N,07956.297,W,052140,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.399,N,07956.360,W,052142,A*3F";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.384,N,07956.423,W,052144,A*35";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.369,N,07956.486,W,052146,A*3B";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.354,N,07956.550,W,052148,A*31";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.339,N,07956.613,W,052150,A*37";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.324,N,07956.676,W,052152,A*3A";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.309,N,07956.739,W,052154,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.294,N,07956.803,W,052156,A*38";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.279,N,07956.866,W,052158,A*36";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.264,N,07956.929,W,052200,A*3E";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.249,N,07956.992,W,052202,A*33";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.234,N,07957.056,W,052204,A*3F";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.218,N,07957.119,W,052206,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.203,N,07957.182,W,052208,A*3F";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.188,N,07957.245,W,052210,A*3E";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.173,N,07957.309,W,052212,A*31";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.158,N,07957.372,W,052214,A*32";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.144,N,07957.435,W,052216,A*39";
			nmea(gpsdata);

			gpsdata="$GPGLL,4338.129,N,07957.498,W,052218,A*3B";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,76.0,M,32.5,M,82.7,M*16";
			nmea(gpsdata);

			gpsdata="$PGRMZ,111,f,2*1B";
			nmea(gpsdata);

			gpsdata="$GPRTE,1,1,c,*37";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,38,209,00*7D";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,43,139,45,19,20,286,00,21,59,095,46,22,21,169,38*7E";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,48,29,24,051,41,37,17,179,00,39,16,175,00*78";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5806,N,02056.5090,E,202616,A,A*42";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.1,M,69.9,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,120,f,2*19";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,38,209,00*7D";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,43,139,45,19,20,286,00,21,59,095,46,22,21,169,38*7E";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,48,29,24,051,41,37,17,179,00,39,16,175,00*78";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5807,N,02056.5092,E,202618,A,A*4F";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.1,M,69.9,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,127,f,2*1E";
			nmea(gpsdata);

			gpsdata="$GPRTE,1,1,c,*37";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,37,209,00*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,43,139,45,19,20,286,00,21,59,095,45,22,21,169,38*7D";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,47,29,24,051,41,37,17,179,00,39,16,175,00*77";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5807,N,02056.5096,E,202620,A,A*40";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.1,M,69.9,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,132,f,2*1A";
			nmea(gpsdata);

			gpsdata="$GPRTE,1,1,c,*37";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,37,209,00*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,43,139,45,19,20,286,00,21,59,095,45,22,21,169,37*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,47,29,24,051,41,37,17,179,00,39,16,175,00*77";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5808,N,02056.5100,E,202622,A,A*43";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.1,M,69.9,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,137,f,2*1F";
			nmea(gpsdata);

			gpsdata="$GPRTE,1,1,c,*37";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,37,209,00*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,43,139,45,19,20,286,00,21,59,095,45,22,21,169,37*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,47,29,24,051,41,37,17,179,00,39,16,175,00*77";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5809,N,02056.5106,E,202624,A,A*42";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.1,M,69.9,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,142,f,2*1D";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,37,209,00*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,43,139,45,19,20,286,00,21,59,095,45,22,21,169,36*73";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,47,29,24,051,41,37,17,179,00,39,16,175,00*77";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5809,N,02056.5113,E,202626,A,A*44";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.1,M,69.9,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,148,f,2*17";
			nmea(gpsdata);

			gpsdata="$GPRTE,1,1,c,*37";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,37,209,00*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,44,138,45,19,20,286,00,21,59,095,45,22,21,169,35*76";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,47,29,24,051,41,37,17,179,00,39,16,175,00*77";
			nmea(gpsdata);

			gpsdata="$GPGLL,6445.5809,N,02056.5121,E,202628,A,A*4B";
			nmea(gpsdata);

			gpsdata="$GPBOD,,T,,M,,*47";
			nmea(gpsdata);

			gpsdata="$PGRME,65.6,M,24.0,M,69.8,M*1A";
			nmea(gpsdata);

			gpsdata="$PGRMZ,153,f,2*1D";
			nmea(gpsdata);

			gpsdata="$GPRTE,1,1,c,*37";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,1,12,03,51,271,00,08,15,353,00,15,59,213,00,16,37,209,00*72";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,2,12,18,44,138,45,19,20,286,00,21,59,095,44,22,21,169,34*76";
			nmea(gpsdata);

			gpsdata="$GPGSV,3,3,12,26,22,063,47,29,24,051,41,37,17,179,00,39,16,175,00*77";
			nmea(gpsdata);
			//------------------------------------------------------------------------------
			
			nmea("$GPRMC,180106,A,6445.5923,N,02056.5368,E,0.0,315.9,121004,6.1,E,A*1D");
			nmea("$GPRMB,A,,,,,,,,,,,,A,A*0B");
			nmea("$GPGGA,180106,6445.5923,N,02056.5368,E,1,05,3.7,47.0,M,24.3,M,,*7C");
			nmea("$GPGSA,A,3,06,10,,,17,,,24,,30,,,3.9,3.7,1.0*39");

			//------------------------------------------------------------------------------
		}

		public void Start()
		{
			if (state!=States.Stopped) return;

			ThreadStart runner = new ThreadStart(run);
			thrd = new Thread(runner);
			thrd.Priority=ThreadPriority.BelowNormal;
			thrd.Start();
		}

		public void Stop()
		{
			if (state!=States.Running) return;
			setstate=States.Stopping;
		}

		#endregion

		#region protected methods
		protected virtual void OnGpsSentence(GpsSentenceEventArgs e)
		{
			if (GpsSentence != null) GpsSentence(this, e);
		}

		protected virtual void OnGpsCommState(GpsCommStateEventArgs e)
		{
			if (GpsCommState!=null) GpsCommState(this, e);
		}

		protected virtual void OnGpsStatus(GpsStatusEventArgs e)
		{
			if (GpsStatus!=null) GpsStatus(this, e);
		}

		protected virtual void OnFix(objPosition pos,FixType fixtype)
		{
			if (Fix!=null) Fix(this,pos,fixtype);
		}

		protected virtual void OnMovement(objMovement mov)
		{
			if (Movement!=null) Movement(this,mov);
		}

		protected virtual void OnSatellites(objSatellite[] satellites)
		{
			if (Satellite!=null) Satellite(this,satellites);
		}

		protected virtual void OnError(Exception exception,string message,string gps_data)
		{
			lasterror=message;
			if (Error!=null) Error(this,exception,message,gps_data);
		}

		protected virtual void OnAutoDiscovery(GpsAutoDiscoveryEventArgs e)
		{
			if (GpsAutoDiscovery!=null) GpsAutoDiscovery(this, e);
		}
		#endregion

		#region events
		public event GpsSentenceEventHandler GpsSentence;
		public event GpsCommStateEventHandler GpsCommState;
		public event GpsStatusEventHandler GpsStatus;
		public event FixEventHandler Fix;
		public event MovementEventHandler Movement;
		public event SatelliteEventHandler Satellite;
		public event ErrorEventHandler Error;
		public event GpsAutoDiscoveryEventHandler GpsAutoDiscovery;
		#endregion

		#region delegates
		public delegate void GpsSentenceEventHandler(object sender, GpsSentenceEventArgs e);
		public delegate void GpsCommStateEventHandler(object sender, GpsCommStateEventArgs e);
		public delegate void GpsStatusEventHandler(object sender, GpsStatusEventArgs e);
		public delegate void FixEventHandler(object sender,objPosition pos,FixType fixtype);
		public delegate void MovementEventHandler(object sender,objMovement mov);
		public delegate void SatelliteEventHandler(object sender,objSatellite[] satellites);
		public delegate void ErrorEventHandler(object sender,Exception exception,string message,string gps_data);
		public delegate void GpsAutoDiscoveryEventHandler(object sender, GpsAutoDiscoveryEventArgs e);
		#endregion

		#region properties

		public bool AutoDiscovery
		{
			set
			{
				autodiscovery=value;
			}
		}

		public objSatellite[] Satellites
		{
			get
			{
				return satellites;
			}
		}

		public objPosition Pos
		{
			get
			{
				return pos;
			}
		}

		private objMovement Mov
		{
			get
			{
				return mov;
			}
		}

		public bool TraceOn
		{
			set
			{
				traceon=value;
			}
			get
			{
				return traceon;
			}
		}

		public bool DemoModeOn
		{
			set
			{
				demomodeon=value;
			}
		}

		public string DemoFile
		{
			set
			{
				demofile=value;
			}
			get
			{
				return demofile;
			}
		}

		public string TraceFile
		{
			set
			{
				tracefile=value;
			}
			get
			{
				return tracefile;
			}

		}

		public States State 
		{
			get
			{
				return state;
			}
		}

		public StatusType GpsState
		{
			get
			{
				return gpsstatus;
			}
		}

		private States setstate
		{
			set
			{
				state=value;
				this.OnGpsCommState(new GpsCommStateEventArgs(value));
			}
		}


		public string ComPort
		{
			set
			{
				comport=value;
			}
		}

		public Serial.BaudRates BaudRate
		{
			set
			{
				baudrate=value;
			}
		}

		public string LastError
		{
			get
			{
				return lasterror;
			}
		}

		public Decimal HDPOMaxError
		{
			set
			{
				hdopmaxerror=value;
			}
			get
			{
				return hdopmaxerror;
			}
		}

		#region waypoint stuff - not yet fully implemented
		public objPosition WayPoint
		{
			get
			{
				return waypoint;
			}
			set
			{
				waypoint=value;
			}
		}

		public decimal DistanceToWayPoint
		{
			get
			{
				// HELP REQUIRED HERE - to work out distance to waypoint
				calculate_waypoint();
				return distancetowaypoint;
			}
		}
		public decimal BearingToWayPoint
		{
			
			get
			{
				// HELP REQUIRED HERE - to work out distance to waypoint
				calculate_waypoint();
				return bearingtowaypoint;
			}
		}

		public decimal CourseCorrection
		{
			
			get
			{
				// HELP REQUIRED HERE - to work out what course correction to apply to get to waypoint
				calculate_waypoint();
				return coursecorrection;
			}
		}

		#endregion


		#endregion

		#region private methods


		private void reset_gps_vars()
		{
			strreceived="";
			sentencecount=0;
			storedgprmc="";
			storedgpgga="";
			storedgpsa="";
			storedgpgsv="";
			storedgpgll="";
			pos = new objPosition();
			mov = new objMovement();
			satellites = null;
			lasterror="";

			distancetowaypoint=0;
			bearingtowaypoint=0;
			coursecorrection=0;
			
			// Position dilution of precision
			pdop=0;

			// Horizontal dilution of precision
			hdop=0;

			// Vertical dilution of precision
			hdop=0;
		}

		// loads demo data into our nmea parser
		private void load_demo_data()
		{
			if (!File.Exists(demofile))
				return;

			StreamReader demostream = File.OpenText(demofile);
			string str="";
			while ((str=demostream.ReadLine()) != null)
				nmea(str);
		}

		private void calculate_waypoint()
		{
			if (pos==null || waypoint==null) return;
	
			distancetowaypoint=Misc.CalculateDistance(pos,waypoint);
			//bearingtowaypoint= ?;
			//coursecorrection= ?;
		}

		
		private void run()
		{

			if (!demomodeon)
			{
				if (this.autodiscovery)
				{
					setstate=States.AutoDiscovery;
					if (!autodiscover())
					{
						setstate=States.Stopped;
						return;
					}
				}
				
			
			}
			setstate=States.Opening;
			this.reset_gps_vars();
			
			

			if (!demomodeon)
			{
				if (!isconnecteddevice())
				{
					OnError(null,"Com Port "+comport+" Is Not On Device","");
					setstate=States.Stopped;
					return;
				}
				DetailedPortSettings portSettings = new HandshakeNone(); 
				cp = new Port(comport,portSettings);
				cp.Settings.BaudRate=this.baudrate;
				cp.RThreshold=64;
				cp.InputLen=64;

				try 
				{
					cp.Open();
				}
				catch (Exception e)
				{
					OnError(e,"Could Not Open Com Port "+comport,"");
					cp.Dispose();
					setstate=States.Stopped;
					return;
				}
				cp.OnError+=new Port.CommErrorEvent(cp_OnError);
				cp.DataReceived+=new Port.CommEvent(cp_DataReceived);
			}

			if (state==States.Opening)
				setstate=States.Running;

			if (demomodeon)
			{
				load_demo_data();
			}
			else
			{
				while (state==States.Running)
				{
					Thread.Sleep(10);
				}

				if (cp.IsOpen) cp.Close();
				cp.Dispose();
				cp=null;
			}
			setstate=States.Stopped;
		}

	/// <summary>
	/// this is fired when the comport receives data
	/// </summary>
		private void cp_DataReceived()
		{
			if (state!=States.Running) return;


			byte[] inputData = new byte[cp.InputLen];

			inputData = cp.Input;

			string strret = Encoding.ASCII.GetString(inputData, 0,inputData.Length);

			// belt and braces - double check if our received data is over 200 chars. trash the first part
			if (strreceived.Length>200)
			{
				strreceived=strreceived.Substring(200);
			}

			this.strreceived+=strret;

			int intcrl=strreceived.IndexOf("\r\n");
			if (intcrl==-1) return;


			string strdata=strreceived.Substring(0,intcrl);
			// is this the last 2 chars ?
			if (intcrl+2==strreceived.Length)
				strreceived="";
			else
				strreceived=strreceived.Substring(intcrl+2);

			nmea(strdata);
		}



		/// <summary>
		/// processes our string of nmea data
		/// </summary>
		/// <param name="gps_data">string of input data</param>
		/// <returns>Boolean to indicate if we were able to process string of data ok</returns>
		private bool nmea(string gps_data) 
		{
			// GPS data can't be zero length
			if (gps_data.Length==0) return false;

			// GPS data can't be longer than 82 character
			if (gps_data.Length>82) return false;

			// first character must be a $
			if (gps_data[0]!='$') return false;

			// remove our leading character
			string strdata=gps_data.Substring(1);

			// see if the last block contains a * used to see if we have a checksum
			int intstarpos=strdata.IndexOf('*');
			if (intstarpos>=0)
			{
				// we have a checksum so check it...
				string strchecksum=strdata.Substring(intstarpos+1);

				// remove checksum from end of string
				strdata=strdata.Substring(0,strdata.Length-strchecksum.Length-1);

				if (!checksum(strdata,strchecksum))
				{
					OnError(null,"Checksum failed on: '"+gps_data+"'",gps_data);
					return false;
				}
			}

			String[] strrecarray = strdata.Split(',');

			// get the first block which is the sentance id
			string strsentance=strrecarray[0];

			ArrayList sbdata = new ArrayList();

			// get the data block minus the first block
			for (int i=1;i<strrecarray.Length;i++)
				sbdata.Add(strrecarray[i]);

			// if all is well raise our main GPS event
			string[] arrydata = (string[]) sbdata.ToArray(typeof(string));

			// incremenet our counter
			if (sentencecount==int.MaxValue) sentencecount=0;

			sentencecount++;
			OnGpsSentence(new GpsSentenceEventArgs(gps_data,sentencecount));

			if (traceon)
			{
				StreamWriter sw = File.AppendText(tracefile);
				sw.WriteLine(gps_data);
				sw.Close();
			}

			// simple checksums to see if we need to fire and event
			decimal poscheck = pos.Latitude_Fractional + pos.Longitude_Fractional;
			decimal movcheck = mov.SpeedKnots+mov.Track;
			StatusType oldgpsstatus = this.gpsstatus;
			switch (strsentance)
			{
				case "GPRMC": // Recommended minimum specific GPS/Transit data 
					if (storedgprmc!=gps_data)
					{
						storedgprmc=gps_data;
						process_gprmc(arrydata,gps_data);
					}
					break;
				case "GPGGA": //Global positioning system fixed data
					if (storedgpgga!=gps_data)
					{
						storedgpgga=gps_data;
						process_gpgga(arrydata,gps_data);
					}
					break;

				case "GPGSA": // GPS DOP and active satellites
					if (storedgpsa!=gps_data)
					{
						storedgpsa=gps_data;
						process_gpgsa(arrydata,gps_data);
					}
					break;

				case "GPGSV": // Satellites in view
					if (storedgpgsv!=gps_data)
					{
						storedgpgsv=gps_data;
						process_gpgsv(arrydata,gps_data);
						OnSatellites(this.satellites);
					}
					break;
				case "GPGLL": // Location Fix
					if (storedgpgll!=gps_data)
					{
						storedgpgll=gps_data;
						process_gpgll(arrydata,gps_data);
					}
					break;
				case "GPVTG": // 
					if (storedgpvtg!=gps_data)
					{
						storedgpvtg=gps_data;
						process_gpvtg(arrydata,gps_data);
					}
					break;	
				default:
					return true;

			}

			// see if our long and lat have changed
			if (poscheck != (pos.Latitude_Fractional  + pos.Longitude_Fractional ))
			{
				// only fire if we have under the HDOP maxium error
				if (hdop<=hdopmaxerror)
					OnFix(pos,fixtype);
			}

			if (movcheck!=mov.SpeedKnots+mov.Track)
			{
				// only fire if we have under the HDOP maxium error
				if (hdop<=hdopmaxerror)
					OnMovement(mov);
			}

			if (oldgpsstatus!=gpsstatus)
			{
				OnGpsStatus(new GpsStatusEventArgs(gpsstatus));
			}
			return true;

		}

		/// <summary>
		/// this checksums all nmea sequences
		/// </summary>
		/// <param name="strtocheck">string to check</param>
		/// <param name="strchecksum">checksum</param>
		/// <returns>true if checksum computes</returns>
		private bool checksum(string strtocheck,string strchecksum)
		{
			int intor=0;
			// go from first character upto last *
			for(int i=0;(i<strtocheck.Length);i++)
				intor=intor^ (int) (strtocheck[i]);

			int y = 0;

			try 
			{
				y = Convert.ToInt32(strchecksum,16);
			}
			catch
			{
				return false;
			}
			if (intor != y)
			{
				// debug for checksum failures
				intor+=0;
			}
			return (intor == y);
		}


		private void cp_OnError(string Description)
		{
			if (state!=States.Running) return;
			OnError(null,"Com Port Error Received "+Description,"");
			this.setstate=States.Stopping;
		}

		private bool isconnecteddevice()
		{
			
			foreach (string port in this.devices())
			{
			
				if (port.ToUpper()==this.comport.ToUpper())
				{
					return true;
				}
			}
			return false;
		}
		private string[] devices()
		{
			// lists all modems on the device
			ArrayList al = new ArrayList();
			RegistryKey defkey;
			string[] keyNames;
			string keyvalue="";
			string strport="";
		
			// now get the active
			defkey=Registry.LocalMachine.CreateSubKey(@"Drivers\Active");
			
			keyNames =defkey.GetSubKeyNames();
			foreach (string x in keyNames)
			{
				
				try	
				{
					keyvalue=defkey.CreateSubKey(x).GetValue("Key").ToString();
					strport=defkey.CreateSubKey(x).GetValue("Name").ToString();
					
					// fudge to remove rubbish off the end of the string
					strport = strport.Substring(0,strport.IndexOf("'"));

					RegistryKey thekey=Registry.LocalMachine.CreateSubKey(keyvalue);
					
					if (strport.StartsWith("COM"))
						al.Add(strport);
				}
				catch
				{

				}
			}

			return (string[]) al.ToArray(typeof(string));
		}

		/// <summary>
		/// Auto discover our GPS
		/// </summary>
		/// <returns>True if we have found a GPS</returns>
		private bool autodiscover()
		{
			Port cp;
			// get a list of devices
			foreach (string port in this.devices())
			{
				OnAutoDiscovery(new GpsAutoDiscoveryEventArgs("Testing "+port));
				
				cp = new Port(port);
				try
				{
					cp.Open();
				}
				catch
				{
					OnAutoDiscovery(new GpsAutoDiscoveryEventArgs("Failed To Open "+port));
				
				}
		
				if (cp.IsOpen)
				{
					// test for gps data
					OnAutoDiscovery(new GpsAutoDiscoveryEventArgs("Opened "+port));
					// loop around the baud rates to see if anything is being received
					cp.DetailedSettings.BasicSettings.BaudRate=BaudRates.CBR_19200;

					// is nema being received ?

					// TODO HELP REQUIRED HERE

					
					cp.Close();
				}
					cp.Dispose();
					cp=null;
			}
			OnAutoDiscovery(new GpsAutoDiscoveryEventArgs("No GPS Detected"));
			return false;
		}

		#endregion

		#region gps stored data
		private StatusType gpsstatus=StatusType.Warning; 
		private objPosition pos;
		private objMovement mov;
		private FixType fixtype=FixType.NotSet;
		private objSatellite[] satellites = null;
		#endregion 

		#region gpsmessage handlers
		private void process_gpgsv(string []strdata, string gpsdata)
		{ 
			try
			{
				int numberofmessages=0;
				int messagenumber=0;
				int numsat=0;
				// 0 Number of messages 3 Number of messages in complete message (1-3)
				if (strdata[0].Length>0)
				{
					numberofmessages=Convert.ToInt32(strdata[0]);
				}

				// 1 Sequence number 1 -Sequence number of this entry (1-3)
				if (strdata[1].Length>0)
				{
					messagenumber=Convert.ToInt32(strdata[1]);
				}
				// 2 Satellites in view - 10
				if (strdata[2].Length>0)
				{
					numsat=Convert.ToInt32(strdata[2]);
					if (satellites==null)
					{
						setupsats(numsat);
					}
					else
					{
						if (satellites.Length!=numsat)
						{
							setupsats(numsat);
						}
					}
				}

				if (messagenumber==0||numsat==0) return;

				// New version
				int whichsat=(numsat-((messagenumber-1)*4)-1);
				int sats;
				int intcount=0;


				if (whichsat >= 4)
					sats = 4;
				else
					sats = numsat-((messagenumber-1)*4);


				for (int i=1;i<=sats;i++) 
				{
					// 3 Satellite ID 1 - 20- Range is 1-32
					if (strdata[intcount+3].Length>0)
						satellites[whichsat].ID=Convert.ToInt32(strdata[intcount+3]);

					// 4 Elevation 1- 78- Elevation in degrees (0-90)
					if (strdata[intcount+4].Length>0)
						satellites[whichsat].Elevation=Convert.ToInt32(strdata[intcount+4]); 

					// 5 Azimuth 1 - 331- Azimuth in degrees (0-359)
					if (strdata[intcount+5].Length>0)
						satellites[whichsat].Azimuth=Convert.ToInt32(strdata[intcount+5]);

					// 6 SNR 1 -45 - Signal to noise ration in dBHZ (0-99)
					if (strdata[intcount+6].Length>0)
						satellites[whichsat].SNR=Convert.ToInt32(strdata[intcount+6]);
					else
						satellites[whichsat].SNR=0;

					intcount += 4;
					whichsat--;
				}
			}
			catch(Exception ex)
			{
				OnError(ex,"Error in process_gpgsv",gpsdata);
			}
		}


		private void process_gpgsa(string []strdata,string gpsdata)
		{
			try
			{
				//0 Mode 1 - A - A = Auto 2D/3D, M = Forced 2D/3D

				//1 Mode 1 - 3 - 1 = No fix, 2 = 2D, 3 = 3D
				if (strdata[1].Length>0)
				{
					switch (Convert.ToInt32(strdata[1]))
					{
						case 1:
							fixtype=FixType.NotSet;
							break;
						case 2:
							fixtype=FixType.NoAltitude;
							break;
						case 3:
							fixtype=FixType.WithAltitude;
							break;
						default:
							fixtype=FixType.NotSet;
							break;
					}
				}

				if (satellites!=null)
				{
					foreach (objSatellite s in this.satellites)
					{
						s.Active=false;
						s.Channel=0;
					}



					// 2 Satellite used 1 - 01 - Satellite used on channel 1
					// 3 Satellite used 1 - 01 - Satellite used on channel 2
					// 4 Satellite used 1 - 01 - Satellite used on channel 3
					// 5 Satellite used 1 - 01 - Satellite used on channel 4
					// 6 Satellite used 1 - 01 - Satellite used on channel 5
					// 7 Satellite used 1 - 01 - Satellite used on channel 6
					// 8 Satellite used 1 - 01 - Satellite used on channel 7
					// 9 Satellite used 1 - 01 - Satellite used on channel 8
					// 10 Satellite used 1 - 01 - Satellite used on channel 9
					// 11 Satellite used 1 - 01 - Satellite used on channel 10
					// 12 Satellite used 1 - 01 - Satellite used on channel 11
					// 13 Satellite used 1 - 01 - Satellite used on channel 12

					for (int intcount=0;intcount<12;intcount++)
					{
						string strid=strdata[2+intcount];
						if (strid!="")
							setsat(Convert.ToInt32(strid),intcount+1);
					}
				}

				// 14 PDOP 40.4 Position dilution of precision
				if (strdata[14].Length>0)
					pdop=Misc.ToDecimal(strdata[14]);
		
				// 15 HDOP 24.4 Horizontal dilution of precision
				if (strdata[15].Length>0)
					hdop=Misc.ToDecimal(strdata[15]);
					
				
				// 16 VDOP 32.2 Vertical dilution of precision
				if (strdata[16].Length>0)
					vdop=Misc.ToDecimal(strdata[16]);
		
				
				
			}
			catch(Exception ex)
			{
				OnError(ex,"Error in process_gpgsa",gpsdata);
			}
		}


		/// <summary>
		/// sets the id's in our list of satellites
		/// </summary>
		/// <param name="intcount"></param>
		/// <param name="id"></param>
		private void setsat(int id,int channel)
		{
			if (id==0) return;

			foreach (objSatellite s in this.satellites)
			{
				if (s.ID==id)
				{
					s.Active=true;
					s.Channel=channel;
					break;
				}
			}

		}
		/// <summary>
		/// sets up our array of sats
		/// </summary>
		/// <param name="numsats"></param>
		private void setupsats(int numsats)
		{
			satellites = new objSatellite[numsats];

			for (int intcount=0;intcount<satellites.Length;intcount++)
			{
				if (satellites[intcount]==null)
					satellites[intcount]= new objSatellite();
			}
		}

		private void process_gpgga(string []strdata,string gpsdata)
		{
			try
			{
				//0 UTC Time
				// New 31/01/05
				if (strdata[0].Length>0)
				{
					// Problem : it's SAT Time but PocketPC date
					// Time and date must be separated
					settimeonly(strdata[0]);
				}
				//1 Latitude
				// New 31/01/05
				if (strdata[1].Length>0)
				{
					pos.Latitude_Fractional=Misc.ToDecimal(strdata[1]);
				}

				//2 N/S Indicator
				// New 31/01/05
				if (strdata[2].Length>0)
				{
					if (strdata[2][0]=='N')
						pos.DirectionLatitude=CardinalDirection.North;
					else
						pos.DirectionLatitude=CardinalDirection.South;
				}
				//3 Longitude
				// New 31/01/05
				if (strdata[3].Length>0)
				{
					pos.Longitude_Fractional=Misc.ToDecimal(strdata[3]);
				}
				//4 E/W Indicator
				// New 31/01/05
				if (strdata[4].Length>0)
				{
					if (strdata[4][0]=='W')
						pos.DirectionLongitude=CardinalDirection.West;
					else
						pos.DirectionLongitude=CardinalDirection.East;
				}
				//5 Position Fix - 0 = Invalid, 1 = Valid SPS, 2 = Valid DGPS, 3 = Valid PPS
				// New 31/01/05
				if (strdata[5].Length>0)
				{
					switch (Convert.ToInt32(strdata[5]))
					{
						case 0:
							// ??
							fixtype=FixType.NotSet;
							break;
						case 1:
							// ??
							fixtype=FixType.WithAltitude;
							break;
						case 2:
							// ??
							fixtype=FixType.WithAltitude;
							break;
						case 3:
							// ??
							fixtype=FixType.WithAltitude;
							break;
						default:
							// ??
							fixtype=FixType.NotSet;
							break;
					}
				}
				//6 Satellites Used (0-12)
				// New 31/01/05
				//if (strdata[6].Length>0)
				//{
				// Not necessary
				//}
				//7 HDOP- Horizontal dilution of precision
				// New 31/01/05
				//if (strdata[7].Length>0)
				//{
				// Not necessary
				//}
				//8 Altitude- Altitude in meters according to WGS-84 ellipsoid
				if (strdata[ 8 ].Length>0)
				{
					// New 31/01/05
					pos.Altitude=Misc.ToDecimal(strdata[ 8 ]);
				}
				//9 Altitude Units - M = Meters
				//if (strdata[9].Length>0)
				//{
				// Not necessary
				//}
				//10 Geoid Seperation- Geoid seperation in meters according to WGS-84 ellipsoid
				// New 31/01/05
				if (strdata[10].Length>0)
				{
					pos.GeoidSeparation = Misc.ToDecimal(strdata[10]);
				}
				//11 Geoid Seperation Units - M = Meters
				//if (strdata[11].Length>0)
				//{
				// Not necessary
				//}
				//12 DGPS Age - Age of DGPS data in seconds
				//if (strdata[12].Length>0)
				//{
				// To be implemented
				//}
				//13 DGPS Station ID-0000
				//if (strdata[13].Length>0)
				//{
				// To be implemented
				//}
			}
			catch(Exception ex)
			{
				OnError(ex,"Error in process_gpgga",gpsdata);
			}
		}


		private void process_gpgll(string []strdata,string gpsdata)
		{
			try
			{
				// 0 Latitude 4250.5589 ddmm.mmmm 
				if (strdata[0].Length>0)
				{
					pos.Latitude_Fractional=Misc.ToDecimal(strdata[0]);
				}

				// 1 N/S Indicator S N = North, S = South 
				if (strdata[1].Length>0)
				{
					if (strdata[1][0]=='N')
						pos.DirectionLatitude=CardinalDirection.North;
					else
						pos.DirectionLatitude=CardinalDirection.South;
				}

				// 2 Longitude 14718.5084 dddmm.mmmm 
				if (strdata[2].Length>0)
				{
					pos.Longitude_Fractional=Misc.ToDecimal(strdata[2]);
				}

				// 3 E/W Indicator E E = East, W = West
				if (strdata[3].Length>0)
				{
					if (strdata[3][0]=='W')
						pos.DirectionLongitude=CardinalDirection.West;
					else
						pos.DirectionLongitude=CardinalDirection.East;
				}


				// 4 UTC Time HHMMSS 
				/*
				if (strdata[4].Length>0)
				{
					settime(strdata[4]);
				}
				*/

				// 5 Status A A = Valid, V = Invalid 
				gpsstatus=(strdata[5]=="A")?StatusType.OK:StatusType.Warning;
			}
			catch(Exception ex)
			{
				OnError(ex,"Error in process_gpgll",gpsdata);
			}
		}

		private void process_gprmc(string []strdata,string gpsdata)
		{
			try
			{
				// 0 180432 UTC of position fix in hhmmss format (18 hours, 4 minutes and 32 seconds)
				if (strdata[0].Length>0 && strdata[ 8 ].Length>0)
				{
					settime(strdata[0],strdata[ 8 ]);
				}

				// 1 A Status (A - data is valid, V - warning) 
				gpsstatus=(strdata[1]=="A")?StatusType.OK:StatusType.Warning;
				//2 4027.027912 Geographic latitude in ddmm.mmmmmm format (40 degrees and 27.027912 minutes) 
				if (strdata[2].Length>0)
				{
					pos.Latitude_Fractional=Misc.ToDecimal(strdata[2]);
				}		
			

				// 3 N Direction of latitude (N - North, S - South) 
				if (strdata[3].Length>0)
				{
					if (strdata[3][0]=='N')
						pos.DirectionLatitude=CardinalDirection.North;
					else
						pos.DirectionLatitude=CardinalDirection.South;
				}

				// 4 08704.857070 Geographic longitude in dddmm.mmmmmm format (87 degrees and 4.85707 minutes) 
				if (strdata[4].Length>0)
				{
					pos.Longitude_Fractional=Misc.ToDecimal(strdata[4]);
				}

				// 5 W Direction of longitude (E - East, W - West) 

				if (strdata[5].Length>0)
				{
					if (strdata[5][0]=='W')
						pos.DirectionLongitude=CardinalDirection.West;
					else
						pos.DirectionLongitude=CardinalDirection.East;
				}

				//6 000.04 Speed over ground (0.04 knots) 
				if (strdata[6].Length>0)
					mov.SpeedKnots=Misc.ToDecimal(strdata[6]);


				// 7 181.9 Track made good (heading) (181.9) 
				if (strdata[7].Length>0)
					mov.Track=Misc.ToDecimal(strdata[7]);

				//8 131000 Date in ddmmyy format (October 13, 2000) 
				// used above already

				// 9 1.8 Magnetic variation (1.8) 
				if (strdata[9].Length>0)
					mov.MagneticVariation=Misc.ToDecimal(strdata[9]);

				//10 W Direction of magnetic variation (E - East, W - West) 
				if (strdata[10].Length>0)
				{
					if (strdata[10][0]=='W')
						mov.DirectionMagnetic=CardinalDirection.West;
					else
						mov.DirectionMagnetic=CardinalDirection.East;
				}


				// 11 D Mode indication (A - autonomous, D - differential, N - data not valid) 
			}
			catch(Exception ex)
			{
				OnError(ex,"Error in process_gprmc",gpsdata);
			}
		}

		private void process_gpvtg(string []strdata,string gpsdata)
		{
			try
			{
				// 0 309.62 degrees Course over ground - Measured heading
				if (strdata[0].Length>0)
				{
					mov.Track=Misc.ToDecimal(strdata[0]);
				}

				// 1 T Course Reference - True
				if (strdata[1].Length>0)
				{
					//
				}
 
				//2 xxx.xx degrees Course over ground - Measured heading 
				if (strdata[2].Length>0)
				{
					//
				}		
			
				// 3 T Course Reference - Magnetic
				if (strdata[3].Length>0)
				{
					
				}

				// 4 0.13 Measured horinzontal speed in knots 
				if (strdata[4].Length>0)
				{
					mov.SpeedKnots=Misc.ToDecimal(strdata[4]);
				}

				// 5 N for knots 
				if (strdata[5].Length>0)
				{
					//
				}

				// 6 0.2 Measured horinzontal speed in kilometers per hours 
				if (strdata[6].Length>0)
				{
					//mov.SpeedKph = Misc.ToDecimal(strdata[6]);
				}

				// 7 K for kilometers per hours 
				if (strdata[7].Length>0)
				{
					//
				}

			}
			catch(Exception ex)
			{
				OnError(ex,"Error in process_gpvtg",gpsdata);
			}
		}


		private void settimeonly(string strtime)
		{
			int utchours = Convert.ToInt32(strtime.Substring(0, 2));
			int utcminutes = Convert.ToInt32(strtime.Substring(2, 2));
			int utcseconds = Convert.ToInt32(strtime.Substring(4, 2));
			int utcmilliseconds =0;
			// extract milliseconds if it is available
			if (strtime.Length > 7)
				utcmilliseconds = Convert.ToInt32(strtime.Substring(7));
			
			DateTime today = DateTime.Now.ToUniversalTime();
			int day = today.Day;
			int month = today.Month;
			int year = today.Year;
			pos.SatTime= new DateTime(year, month, day, utchours, utcminutes, utcseconds, utcmilliseconds);

		}
		private void settime(string strtime,string strdate)
		{
			int utchours = Convert.ToInt32(strtime.Substring(0, 2));
			int utcminutes = Convert.ToInt32(strtime.Substring(2, 2));
			int utcseconds = Convert.ToInt32(strtime.Substring(4, 2));
			int utcmilliseconds =0;

			// extract milliseconds if it is available
			if (strtime.Length > 7)
				utcmilliseconds = Convert.ToInt32(strtime.Substring(7));

			// now build a datetime object with all values
			DateTime today = DateTime.Now.ToUniversalTime();
			int day = Convert.ToInt32(strdate.Substring(0, 2));
			int month = Convert.ToInt32(strdate.Substring(2, 2));
			int year = Convert.ToInt32(strdate.Substring(4, 2));
			pos.SatTime= new DateTime(year, month, day, utchours, utcminutes, utcseconds, utcmilliseconds);
		}

		#endregion

	}
}
