using System;
using OpenNETCF.IO.Serial;
using System.Text;
using System.Collections;
using System.Threading;
using System.IO;
using System.Globalization;

/// <summary>
/// Produced Under the Terms Of The OpennetCF License
/// 
/// By Richard Jones - Richard@BinaryRefinery.com
/// 
/// </summary>
namespace OpenNETCF.IO.Serial.GPS
{

	public class Misc // utility functions
	{
		
		public static decimal ToDecimal(object obin)
		{
			string StringSeparator = System.Globalization.CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
			
			if (StringSeparator == ",")
				return Convert.ToDecimal(obin.ToString().Replace(".",","));
			else
				return Convert.ToDecimal(obin);
		}

		public static decimal KnotsToMph(decimal Knots)
		{
			return Knots * 1.151m;
		}

		public static decimal KnotsToKph(decimal Knots)
		{
			return Knots * 1.852m;
		}

		//TODO grabbed conversion from MSDN editor
		//not sure what the actual unit for altitude is
		public static decimal AltToFeet(decimal Alt)
		{
			return (Alt / 39.36m) * 12;
		}


		public static decimal MetersToFeet(decimal metres)
		{
			return metres * 3.280839895013122m;
		}

		public static decimal MetersToInches(decimal metres)
		{
			return metres * 39.700874015748m;
		}

		public static decimal MetersToYards(decimal metres)
		{
			return metres *  1.0936132983377087m;
		}

		public static decimal MetersToKM(decimal metres)
		{
			return metres * 0.001m;
		}

		public static decimal MetersToCm(decimal metres)
		{
			return metres * 100m;
		}

		public static decimal MetersToMm(decimal metres)
		{
			return metres * 1000m;
		}

		public static decimal MetersToMiles(decimal metres)
		{
			return metres * 0.0006213711922373347m;
		}

		public static decimal KmToMiles(decimal km)
		{
			return km * 0.6213711922373347m;
		}

		public static decimal NmToMeters(decimal nm)
		{
			return nm * 1852m;
		}

		public static decimal CalculateDistance(objPosition pos1,objPosition pos2)
		{
			double lat1 = (double)pos1.Latitude_Decimal;
			double lat2 = (double)pos2.Latitude_Decimal;
			double lon1 = (double)pos1.Longitude_Decimal;
			double lon2 = (double)pos2.Longitude_Decimal;		
			double distance;
	
			if ((lat1 == lat2) && (lon1 == lon2)) 
				return 0;

			double DEG2RAD = Math.PI/180;
			lat1 *= DEG2RAD;
			lat2 *= DEG2RAD;
			lon1 *= DEG2RAD;
			lon2 *= DEG2RAD;
			distance = (60.0 * ((Math.Acos((Math.Sin(lat1) * Math.Sin(lat2)) + (Math.Cos(lat1) * Math.Cos(lat2) * Math.Cos(lon2 - lon1)))) / DEG2RAD));			
			
			return NmToMeters((decimal)distance);
		}


		///summary>
		/// Converts fractional degrees to decimal degrees
		/// </summary>
		/// <param name="decin"></param>
		/// <returns></returns>
		public static decimal FractionalToDecimalDegrees(decimal decin) 
		{
			bool positve = decin>0;
			string dm=Math.Abs(decin).ToString("00000.0000");

			//Get the fractional part of minutes
			int intdelim=dm.IndexOf(CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator);
			
			decimal fm = Misc.ToDecimal("0" + dm.Substring(intdelim));

			//Get the minutes.
			decimal min = Misc.ToDecimal(dm.Substring(intdelim - 2, 2));

			//Degrees
			decimal deg= Misc.ToDecimal(dm.Substring(0, intdelim - 2));

			decimal result = deg + (min + fm) / 60;
			if (positve)
				return result;
			else
				return -result;
				
		}

		//format (dd.dddd) to decimal minute format (ddmm.mmmm).
		//Example 58.65375 => 5839.225 (58 39.225min)
		public static decimal DecimalToFractionalDegrees(decimal dec)
		{
			decimal d=(Decimal.Truncate(dec)); //58
			decimal m=(dec-d)*60; //39.2250 
			m=Math.Round(m,4); //39.2250
			return (d*100)+m; //5839.2250
		}
	
	
		/// <summary>
		/// convert decimal degrees to sexagesimal format
		/// </summary>
		/// <param name="thedata"></param>
		/// <returns></returns>
		/// 
		public static string DecimalToSexagesimal(decimal inputdata)
		{

			// orignal input = the data 4533.3512

			string strdata=Math.Abs(inputdata).ToString("00000.0000");

			// degrees is 45
			string degrees=strdata.Substring(0,3);


			string minutes = strdata.Substring(3,2);
			// minutes is 33

			decimal decseconds = Misc.ToDecimal(strdata.Substring(5));
			// decseconds = .3512

			decseconds = Math.Round(decseconds*60,2);

			// seconds = 21.07

			return degrees+""+minutes+"'"+decseconds.ToString("00.00")+"\"";


		}

	}
}